@@ -51,6 +51,7 @@ class Core
     "-r"  => [ false, "Reset the ring buffer for the session given with -i, or all"    ],
     "-u"  => [ true,  "Upgrade a shell to a meterpreter session on many platforms"     ],
     "-t"  => [ true,  "Set a response timeout (default: 15)"                           ],
+    "-S"  => [ true, "Row search filter."                                              ],
     "-x" =>  [ false, "Show extended information in the session table"                 ])
 
   @@threads_opts = Rex::Parser::Arguments.new(
@@ -85,13 +86,20 @@ class Core
     "-k" => [ true,  "Keep (include) arg lines at start of output."   ],
     "-c" => [ false, "Only print a count of matching lines."          ])
 
+  @@search_opts = Rex::Parser::Arguments.new(
+    "-h" => [ false, "Help banner."                                   ],
+    "-S" => [ true, "Row search filter."                              ])
 
+  @@history_opts = Rex::Parser::Arguments.new(
+    "-h" => [ false, "Help banner."                                   ],
+    "-a" => [ false, "Show all commands in history."                  ],
+    "-n" => [ true,  "Show the last n commands."                      ],
+    "-u" => [ false, "Show only unique commands."                     ])
 
   @@irb_opts = Rex::Parser::Arguments.new(
     "-h" => [ false, "Help banner."                                   ],
     "-e" => [ true,  "Expression to evaluate."                        ])
 
-
   # Returns the list of commands supported by this command dispatcher
   def commands
     {
@@ -105,6 +113,7 @@ class Core
       "getg"       => "Gets the value of a global variable",
       "grep"       => "Grep the output of another command",
       "help"       => "Help menu",
+      "history"    => "Show command history",
       "irb"        => "Drop into irb scripting mode",
       "load"       => "Load a framework plugin",
       "quit"       => "Exit the console",
@@ -133,6 +142,7 @@ class Core
     @cache_payloads = nil
     @previous_module = nil
     @module_name_stack = []
+    @history_limit = 100
   end
 
   #
@@ -165,9 +175,6 @@ class Core
     driver.update_prompt
   end
 
-
-
-
   def cmd_cd_help
     print_line "Usage: cd <directory>"
     print_line
@@ -473,6 +480,55 @@ class Core
 
   alias cmd_quit cmd_exit
 
+  def cmd_history(*args)
+    length = Readline::HISTORY.length
+    uniq   = false
+
+    if length < @history_limit
+      limit = length
+    else
+      limit = @history_limit
+    end
+
+    @@history_opts.parse(args) do |opt, idx, val|
+      case opt
+      when "-a"
+        limit = length
+      when "-n"
+        return cmd_history_help unless val && val.match(/\A[-+]?\d+\z/)
+        if length < val.to_i
+          limit = length
+        else
+          limit = val.to_i
+        end
+      when "-u"
+        uniq = true
+      when "-h"
+        cmd_history_help
+        return false
+      end
+    end
+
+    start   = length - limit
+    pad_len = length.to_s.length
+
+    (start..length-1).each do |pos|
+      if uniq && Readline::HISTORY[pos] == Readline::HISTORY[pos-1]
+        next unless pos == 0
+      end
+      cmd_num = (pos + 1).to_s
+      print_line "#{cmd_num.ljust(pad_len)}  #{Readline::HISTORY[pos]}"
+    end
+  end
+
+  def cmd_history_help
+    print_line "Usage: history [options]"
+    print_line
+    print_line "Shows the command history."
+    print_line "If -n is not set, only the last #{@history_limit} commands will be shown."
+    print @@history_opts.usage
+  end
+
   def cmd_sleep_help
     print_line "Usage: sleep <seconds>"
     print_line
@@ -660,27 +716,34 @@ class Core
   end
 
   def cmd_load_help
-    print_line "Usage: load <path> [var=val var=val ...]"
+    print_line "Usage: load <option> [var=val var=val ...]"
     print_line
-    print_line "Loads a plugin from the supplied path.  If path is not absolute, first looks"
-    print_line "in the user's plugin directory (#{Msf::Config.user_plugin_directory}) then"
-    print_line "in the framework root plugin directory (#{Msf::Config.plugin_directory})."
+    print_line "Loads a plugin from the supplied path."
+    print_line "For a list of built-in plugins, do: load -l"
     print_line "The optional var=val options are custom parameters that can be passed to plugins."
     print_line
   end
 
-  #
-  # Loads a plugin from the supplied path.  If no absolute path is supplied,
-  # the framework root plugin directory is used.
-  #
-  def cmd_load(*args)
-    if (args.length == 0)
-      cmd_load_help
-      return false
+  def list_plugins
+    plugin_directories = {
+      'Framework' => Msf::Config.plugin_directory,
+      'User'      => Msf::Config.user_plugin_directory
+    }
+
+    plugin_directories.each do |type, plugin_directory|
+      items = Dir.entries(plugin_directory).keep_if { |n| n.match(/^.+\.rb$/)}
+      next if items.empty?
+      print_status("Available #{type} plugins:")
+      items.each do |item|
+        print_line("    * #{item.split('.').first}")
+      end
+      print_line
     end
+  end
+
+  def load_plugin(args)
+    path = args[0]
 
-    # Default to the supplied argument path.
-    path = args.shift
     opts  = {
       'LocalInput'    => driver.input,
       'LocalOutput'   => driver.output,
@@ -704,7 +767,6 @@ class Core
         # If the following "path" doesn't exist it will be caught when we attempt to load
         path = Msf::Config.plugin_directory + File::SEPARATOR + plugin_file_name
       end
-
     end
 
     # Load that plugin!
@@ -718,6 +780,21 @@ class Core
     end
   end
 
+  #
+  # Loads a plugin from the supplied path.  If no absolute path is supplied,
+  # the framework root plugin directory is used.
+  #
+  def cmd_load(*args)
+    case args[0]
+    when '-l'
+      list_plugins
+    when '-h', nil, ''
+      cmd_load_help
+    else
+      load_plugin(args)
+    end
+  end
+
   #
   # Tab completion for the load command
   #
@@ -751,10 +828,33 @@ class Core
   end
 
   def cmd_route_help
-    print_line "Usage: route [add/remove/get/flush/print] subnet netmask [comm/sid]"
-    print_line
     print_line "Route traffic destined to a given subnet through a supplied session."
-    print_line "The default comm is Local."
+    print_line
+    print_line "Usage:"
+    print_line "  route [add/remove] subnet netmask [comm/sid]"
+    print_line "  route [add/remove] cidr [comm/sid]"
+    print_line "  route [get] <host or network>"
+    print_line "  route [flush]"
+    print_line "  route [print]"
+    print_line
+    print_line "Subcommands:"
+    print_line "  add - make a new route"
+    print_line "  remove - delete a route; 'del' is an alias"
+    print_line "  flush - remove all routes"
+    print_line "  get - display the route for a given target"
+    print_line "  print - show all active routes"
+    print_line
+    print_line "Examples:"
+    print_line "  Add a route for all hosts from 192.168.0.0 to 192.168.0.0 through session 1"
+    print_line "    route add 192.168.0.0 255.255.255.0 1"
+    print_line "    route add 192.168.0.0/24 1"
+    print_line
+    print_line "  Delete the above route"
+    print_line "    route remove 192.168.0.0/24 1"
+    print_line "    route del 192.168.0.0 255.255.255.0 1"
+    print_line
+    print_line "  Display the route that would be used for the given host or network"
+    print_line "    route get 192.168.0.11"
     print_line
   end
 
@@ -770,10 +870,12 @@ class Core
 
     when "add", "remove", "del"
       subnet = args.shift
-      netmask = nil
-      if subnet
-        subnet, cidr_mask = subnet.split("/")
-        netmask = Rex::Socket.addr_ctoa(cidr_mask.to_i) if cidr_mask
+      subnet,cidr_mask = subnet.split("/")
+      if Rex::Socket.is_ipv4?(args.first)
+        netmask = args.shift
+      else
+        cidr_mask = '32' if cidr_mask.nil?
+        netmask = Rex::Socket.addr_ctoa(cidr_mask.to_i)
       end
 
       netmask = args.shift if netmask.nil?
@@ -835,9 +937,10 @@ class Core
       Rex::Socket::SwitchBoard.flush_routes
 
     when "print"
-      tbl = Table.new(
+      # IPv4 Table
+      tbl_ipv4 = Table.new(
         Table::Style::Default,
-        'Header'  => "Active Routing Table",
+        'Header'  => "IPv4 Active Routing Table",
         'Prefix'  => "\n",
         'Postfix' => "\n",
         'Columns' =>
@@ -852,6 +955,25 @@ class Core
             'Netmask' => { 'MaxWidth' => 17 },
           })
 
+      # IPv6 Table
+      tbl_ipv6 = Table.new(
+        Table::Style::Default,
+        'Header'  => "IPv6 Active Routing Table",
+        'Prefix'  => "\n",
+        'Postfix' => "\n",
+        'Columns' =>
+          [
+            'Subnet',
+            'Netmask',
+            'Gateway',
+          ],
+        'ColProps' =>
+          {
+            'Subnet'  => { 'MaxWidth' => 17 },
+            'Netmask' => { 'MaxWidth' => 17 },
+          })
+
+      # Populate Route Tables
       Rex::Socket::SwitchBoard.each { |route|
         if (route.comm.kind_of?(Msf::Session))
           gw = "Session #{route.comm.sid}"
@@ -859,14 +981,22 @@ class Core
           gw = route.comm.name.split(/::/)[-1]
         end
 
-        tbl << [ route.subnet, route.netmask, gw ]
+        tbl_ipv4 << [ route.subnet, route.netmask, gw ] if Rex::Socket.is_ipv4?(route.netmask)
+        tbl_ipv6 << [ route.subnet, route.netmask, gw ] if Rex::Socket.is_ipv6?(route.netmask)
       }
 
-      if tbl.rows.length == 0
-        print_status('There are currently no routes defined.')
-      else
-        print(tbl.to_s)
+      # Print Route Tables
+      print(tbl_ipv4.to_s) if tbl_ipv4.rows.length > 0
+      print(tbl_ipv6.to_s) if tbl_ipv6.rows.length > 0
+
+      if (tbl_ipv4.rows.length + tbl_ipv6.rows.length) < 1
+        print_status("There are currently no routes defined.")
+      elsif (tbl_ipv4.rows.length < 1) && (tbl_ipv6.rows.length > 0)
+        print_status("There are currently no IPv4 routes defined.")
+      elsif (tbl_ipv4.rows.length > 0) && (tbl_ipv6.rows.length < 1)
+        print_status("There are currently no IPv6 routes defined.")
       end
+
     else
       cmd_route_help
     end
@@ -947,7 +1077,6 @@ class Core
     print_line("Saved configuration to: #{Msf::Config.config_file}")
   end
 
-
   def cmd_spool_help
     print_line "Usage: spool <off>|<filename>"
     print_line
@@ -1012,6 +1141,7 @@ class Core
     script   = nil
     reset_ring = false
     response_timeout = 15
+    search_term = nil
 
     # any arguments that don't correspond to an option or option arg will
     # be put in here
@@ -1024,53 +1154,58 @@ class Core
       # Parse the command options
       @@sessions_opts.parse(args) do |opt, idx, val|
         case opt
-        when '-q'
+        when "-q"
           quiet = true
         # Run a command on all sessions, or the session given with -i
-        when '-c'
+        when "-c"
           method = 'cmd'
           cmds << val if val
-        when '-C'
+        when "-C"
             method = 'meterp-cmd'
             cmds << val if val
-        when '-x'
+        when "-x"
           show_extended = true
-        when '-v'
+        when "-v"
           verbose = true
         # Do something with the supplied session identifier instead of
         # all sessions.
-        when '-i'
+        when "-i"
           sid = val
         # Display the list of active sessions
-        when '-l'
+        when "-l"
           method = 'list'
-        when '-k'
+        when "-k"
           method = 'kill'
           sid = val || false
-        when '-K'
+        when "-K"
           method = 'killall'
         # Run a script on all meterpreter sessions
-        when '-s'
+        when "-s"
           unless script
             method = 'scriptall'
             script = val
           end
         # Upload and exec to the specific command session
-        when '-u'
+        when "-u"
           method = 'upexec'
           sid = val || false
+        # Search for specific session
+        when "-S", "--search"
+          search_term = val
         # Reset the ring buffer read pointer
-        when '-r'
+        when "-r"
           reset_ring = true
           method = 'reset_ring'
         # Display help banner
-        when '-h'
+        when "-h"
           cmd_sessions_help
           return false
-        when '-t'
+        when "-t"
           if val.to_s =~ /^\d+$/
             response_timeout = val.to_i
           end
+        when "-S", "--search"
+          search_term = val
         else
           extra << val
         end
@@ -1187,7 +1322,7 @@ class Core
               session.response_timeout = response_timeout
             end
 
-            output = session.run_cmd cmd
+            output = session.run_cmd(cmd, driver.output)
           end
         end
     when 'kill'
@@ -1336,7 +1471,7 @@ class Core
       end
     when 'list',nil
       print_line
-      print(Serializer::ReadableText.dump_sessions(framework, :show_extended => show_extended, :verbose => verbose))
+      print(Serializer::ReadableText.dump_sessions(framework, :show_extended => show_extended, :verbose => verbose, :search_term => search_term))
       print_line
     end
 
@@ -1448,6 +1583,23 @@ class Core
       end
     end
 
+    # Warn when setting RHOST option for module which expects RHOSTS
+    if args.first.upcase.eql?('RHOST')
+      mod = active_module
+      unless mod.nil?
+        if !mod.options.include?('RHOST') && mod.options.include?('RHOSTS')
+          warn_rhost = false
+          if mod.exploit? && mod.datastore['PAYLOAD']
+            p = framework.payloads.create(mod.datastore['PAYLOAD'])
+            warn_rhost = (p && !p.options.include?('RHOST'))
+          else
+            warn_rhost = true
+          end
+          print_warning("RHOST is not a valid option for this module. Did you mean RHOSTS?") if warn_rhost
+        end
+      end
+    end
+
     # Set the supplied name to the supplied value
     name  = args[0]
     value = args[1, args.length-1].join(' ')
@@ -1604,16 +1756,15 @@ class Core
       return false
     end
 
-    # Walk the plugins array
-    framework.plugins.each { |plugin|
-      # Unload the plugin if it matches the name we're searching for
-      if (plugin.name.downcase == args[0].downcase)
-        print("Unloading plugin #{args[0]}...")
-        framework.plugins.unload(plugin)
-        print_line("unloaded.")
-        break
-      end
-    }
+    # Find a plugin within the plugins array
+    plugin = framework.plugins.find { |p| p.name.downcase == args[0].downcase }
+
+    # Unload the plugin if it matches the name we're searching for
+    if plugin
+      print("Unloading plugin #{args[0]}...")
+      framework.plugins.unload(plugin)
+      print_line("unloaded.")
+    end
   end
 
   #
@@ -1809,7 +1960,6 @@ class Core
 
   alias cmd_unsetg_help cmd_unset_help
 
-
   #
   # Returns the revision of the framework and console library
   #
@@ -2268,7 +2418,6 @@ class Core
     return binary_paths.include? Msf::Config.install_root
   end
 
-
   #
   # Returns an array of lines at the provided line number plus any before and/or after lines requested
   # from all_lines by supplying the +before+ and/or +after+ parameters which are always positive
